@@ -3,6 +3,7 @@
 #include "itemsynctests.h"
 
 #include "common/mimetypes.h"
+#include "common/sleeptimer.h"
 #include "tests/test_utils.h"
 
 #include <QDir>
@@ -36,11 +37,8 @@ public:
 
     void clear()
     {
-        if (isValid()) {
-            for ( const auto &fileName : files() )
-                remove(fileName);
-            m_dir.rmpath(".");
-        }
+        if (isValid())
+            m_dir.removeRecursively();
     }
 
     void create()
@@ -128,13 +126,9 @@ void ItemSyncTests::init()
 {
     TEST(m_test->init());
 
-    // Remove temporary directory.
-    for (int i = 0; i < 10; ++i)
-        TestDir(i, false);
-
     QDir tmpDir(QDir::cleanPath(testDir(0) + "/.."));
     if ( tmpDir.exists() )
-        QVERIFY(tmpDir.rmdir("."));
+        QVERIFY(tmpDir.removeRecursively());
 }
 
 void ItemSyncTests::cleanup()
@@ -389,12 +383,36 @@ void ItemSyncTests::modifyItems()
 
     RUN(args << "keys" << "HOME" << "DOWN" << "F2" << ":XXX" << "F2", "");
     RUN(args << "size", "4\n");
-    RUN(args << "read" << "0" << "1" << "2" << "3", "D,XXX,B,A");
+    RUN(args << "read" << "0" << "1" << "2" << "3" << "4", "D,XXX,B,A,");
 
     file = dir1.file(files[2]);
     QVERIFY(file->open(QIODevice::ReadOnly));
     QCOMPARE(file->readAll().data(), QByteArray("XXX").data());
     file->close();
+
+    const auto script = R"(
+        setCommands([{
+            name: 'Modify current item',
+            inMenu: true,
+            shortcuts: ['Ctrl+F1'],
+            cmd: `
+                copyq: item = selectedItemsData()[0]
+                item[mimeText] = "ZZZ"
+                setSelectedItemData(0, item)
+            `
+        }])
+        )";
+    RUN(script, "");
+    RUN(args << "keys" << "HOME" << "DOWN" << "CTRL+F1", "");
+    WAIT_ON_OUTPUT(args << "read" << "1", "ZZZ");
+    RUN(args << "read" << "0" << "1" << "2" << "3" << "4", "D,ZZZ,B,A,");
+    RUN(args << "unload" << tab1, "");
+    RUN(args << "read" << "0" << "1" << "2" << "3" << "4", "D,ZZZ,B,A,");
+
+    file = dir1.file(files[2]);
+    QVERIFY(file->open(QIODevice::ReadOnly));
+    QCOMPARE(file->readAll().data(), QByteArray("ZZZ").data());
+    file->close();
 }
 
 void ItemSyncTests::modifyFiles()
@@ -741,6 +759,7 @@ void ItemSyncTests::avoidDuplicateItemsAddedFromClipboard()
     const Args args = Args() << "separator" << "," << "tab" << tab1;
 
     RUN("config" << "clipboard_tab" << tab1, tab1 + "\n");
+    WAIT_ON_OUTPUT("isClipboardMonitorRunning", "true\n");
 
     TEST( m_test->setClipboard("one") );
     WAIT_ON_OUTPUT(args << "read(0,1,2,3)", "one,,,");
@@ -751,3 +770,44 @@ void ItemSyncTests::avoidDuplicateItemsAddedFromClipboard()
     TEST( m_test->setClipboard("one") );
     WAIT_ON_OUTPUT(args << "read(0,1,2,3)", "one,two,,");
 }
+
+void ItemSyncTests::saveLargeItem()
+{
+    const auto tab = testTab(1);
+    const auto args = Args("tab") << tab;
+
+    const auto script = R"(
+        write(0, [{
+            'text/plain': '1234567890'.repeat(10000),
+            'application/x-copyq-test-data': 'abcdefghijklmnopqrstuvwxyz'.repeat(10000),
+        }])
+        )";
+    RUN(args << script, "");
+
+    for (int i = 0; i < 2; ++i) {
+        RUN(args << "read(0).left(20)", "12345678901234567890");
+        RUN(args << "read(0).length", "100000\n");
+        RUN(args << "getItem(0)[mimeText].left(20)", "12345678901234567890");
+        RUN(args << "getItem(0)[mimeText].length", "100000\n");
+        RUN(args << "getItem(0)['application/x-copyq-test-data'].left(26)", "abcdefghijklmnopqrstuvwxyz");
+        RUN(args << "getItem(0)['application/x-copyq-test-data'].length", "260000\n");
+        RUN(args << "ItemSelection().selectAll().itemAtIndex(0)[mimeText].length", "100000\n");
+        RUN("unload" << tab, tab + "\n");
+    }
+
+    RUN("show" << tab, "");
+    RUN("keys" << clipboardBrowserId << keyNameFor(QKeySequence::Copy), "");
+    WAIT_ON_OUTPUT("clipboard().left(20)", "12345678901234567890");
+    RUN("clipboard('application/x-copyq-test-data').left(26)", "abcdefghijklmnopqrstuvwxyz");
+    RUN("clipboard('application/x-copyq-test-data').length", "260000\n");
+
+    const auto tab2 = testTab(2);
+    const auto args2 = Args("tab") << tab2;
+    RUN("show" << tab2, "");
+    waitFor(waitMsPasteClipboard);
+    RUN("keys" << clipboardBrowserId << keyNameFor(QKeySequence::Paste), "");
+    RUN(args2 << "read(0).left(20)", "12345678901234567890");
+    RUN(args2 << "read(0).length", "100000\n");
+    RUN(args << "getItem(0)['application/x-copyq-test-data'].left(26)", "abcdefghijklmnopqrstuvwxyz");
+    RUN(args << "getItem(0)['application/x-copyq-test-data'].length", "260000\n");
+}
