@@ -6,7 +6,6 @@
 
 #include <QLocale>
 #include <QString>
-#include <Qt>
 
 #include <algorithm>
 #include <iterator>
@@ -17,10 +16,9 @@ const QLatin1String mimePluginPrefix(COPYQ_MIME_PREFIX "item");
 
 QString escapeHtmlSpaces(const QString &str)
 {
-    QString str2 = str;
-    return str2
-            .replace(' ', "&nbsp;")
-            .replace('\n', "<br />");
+    return QString(str)
+            .replace(' ', QLatin1String("&nbsp;"))
+            .replace('\n', QLatin1String("<br />"));
 }
 
 bool isPluginFormat(const QString &mime)
