@@ -132,18 +132,16 @@ QVariant ClipboardItem::data(int role) const
     switch(role) {
     case Qt::DisplayRole:
     case Qt::EditRole:
-        if ( m_data.contains(mimeText) )
-            return getTextData(m_data);
-        if ( m_data.contains(mimeUriList) )
-            return getTextData(m_data, mimeUriList);
-        break;
+        return getTextData(m_data);
 
     case contentType::data:
         return m_data; // copy-on-write, so this should be fast
     case contentType::hash:
         return dataHash();
     case contentType::hasText:
-        return m_data.contains(mimeText) || m_data.contains(mimeUriList);
+        return m_data.contains(mimeText)
+            || m_data.contains(mimeTextUtf8)
+            || m_data.contains(mimeUriList);
     case contentType::hasHtml:
         return m_data.contains(mimeHtml);
     case contentType::text:
