@@ -14,14 +14,14 @@ mkdir -p "$Destination"
 cmake --install "$BUILD_PATH" --config Release --prefix "$Destination" --verbose
 
 if [[ $WITH_NATIVE_NOTIFICATIONS == ON ]]; then
-    cp -v "$INSTALL_PREFIX/bin/KF5"*.dll "$Destination"
+    cp -v "$INSTALL_PREFIX/bin/KF6"*.dll "$Destination"
     cp -v "$INSTALL_PREFIX/bin/snoretoast.exe" "$Destination"
-    kf5_libraries=(
-        "$Destination/KF5ConfigCore.dll"
-        "$Destination/KF5Notifications.dll"
+    kf_libraries=(
+        "$Destination/KF6ConfigCore.dll"
+        "$Destination/KF6Notifications.dll"
     )
 else
-    kf5_libraries=()
+    kf_libraries=()
 fi
 
 cp -v "$Source/AUTHORS" "$Destination"
@@ -42,11 +42,8 @@ cp -v "$OPENSSL_PATH/$LIBSSL" "$Destination"
 
 "$QTDIR/bin/windeployqt" --help
 "$QTDIR/bin/windeployqt" \
-    --no-system-d3d-compiler \
-    --no-angle \
-    --no-opengl-sw \
-    --no-quick \
-    "${kf5_libraries[@]}" \
+    $WINDEPLOYQT_ARGS \
+    "${kf_libraries[@]}" \
     "$Executable"
 
 # Create and upload portable zip file.
@@ -54,7 +51,7 @@ cp -v "$OPENSSL_PATH/$LIBSSL" "$Destination"
 appveyor PushArtifact "$APP.zip" -DeploymentName "CopyQ Portable"
 
 # This works with minGW, not msvc.
-# objdump -x "$Destination/KF5Notifications.dll" | grep -F "DLL Name"
+# objdump -x "$Destination/KF6Notifications.dll" | grep -F "DLL Name"
 # objdump -x "$Destination/copyq.exe" | grep -F "DLL Name"
 
 # Note: Following removes system-installed dlls to verify required libs are included.
@@ -63,7 +60,11 @@ rm -vf /c/Windows/System32/libssl-*
 rm -vf /c/Windows/SysWOW64/libcrypto-*
 rm -vf /c/Windows/SysWOW64/libssl-*
 OldPath=$PATH
-export PATH=$Destination
+export PATH="$GPGPATH":$Destination
+
+mkdir ~/.gnupg
+chmod go-rwx ~/.gnupg
+gpg --version
 
 export QT_FORCE_STDERR_LOGGING=1
 export COPYQ_TESTS_RERUN_FAILED=1
@@ -74,6 +75,7 @@ export COPYQ_TESTS_RERUN_FAILED=1
 
 # Take screenshots of the app.
 "$Executable" &
+copyq_pid=$!
 "$Executable" showAt 0 0 9999 9999
 
 "$Executable" add "Plain text item"
@@ -121,22 +123,32 @@ screenshot "Screenshot - Tab Menu"
 "$Executable" keys "ESCAPE" "focus:ClipboardBrowser"
 
 "$Executable" exit
-wait
+wait "$copyq_pid"
 
 export PATH=$OldPath
 
 choco install -y InnoSetup
 cmd " /c C:/ProgramData/chocolatey/bin/ISCC.exe /O$APPVEYOR_BUILD_FOLDER /DAppVersion=$APP_VERSION /DRoot=$Destination /DSource=$Source $Source/Shared/copyq.iss"
 
+installer="$APPVEYOR_BUILD_FOLDER/copyq-$APP_VERSION-setup.exe"
+appveyor PushArtifact "$installer" -DeploymentName "CopyQ Setup"
+
 # Test installer
-cmd " /c $APPVEYOR_BUILD_FOLDER/copyq-$APP_VERSION-setup.exe /VERYSILENT /suppressmsgboxes"
-"C:/Program Files (x86)/CopyQ/copyq.exe" --version
+cmd " /c $installer /VERYSILENT /SUPPRESSMSGBOXES"
+"C:/Program Files/CopyQ/copyq.exe" --version
 
 # Test installer can close the app safely
 (
-    sleep 5
-    cmd " /c $APPVEYOR_BUILD_FOLDER/copyq-$APP_VERSION-setup.exe /VERYSILENT"
+    # Wait for CopyQ to start
+    "C:/Program Files/CopyQ/copyq.exe" ""
+    cmd " /c $installer /VERYSILENT /SUPPRESSMSGBOXES"
+    echo "Installation finished"
 ) &
+installer_pid=$!
 export COPYQ_LOG_LEVEL=DEBUG
-"C:/Program Files (x86)/CopyQ/copyq.exe"
-wait
+"C:/Program Files/CopyQ/copyq.exe"
+wait "$installer_pid"
+
+gpgconf --kill all
+
+echo "All OK"
